#!/bin/bash

###### DEFAULTS
## Defaults provided by provision-server.conf
## Per server overridden by ${shorthost}/provision-${shorthost}.conf
set -a
if [[ -e /root/provision/provision-server.conf ]]; then
    echo "Loading \"/root/provision/provision-server.conf\"";
    source /root/provision/provision-server.conf;
fi;
if [[ -e /root/provision/${shorthost}/provision-${shorthost}.conf ]]; then
    echo "Loading \"/root/provision/${shorthost}/provision-${shorthost}.conf\"";
    source /root/provision/${shorthost}/provision-${shorthost}.conf
fi;
set +a
###### DEFAULTS

# Check things, load confs
export shorthost=${1};
if [[ -z ${shorthost} ]]; then
    echo "Must provide shorthost.";
    exit;
fi;

if [[ ! -d /root/provision/${shorthost} ]]; then
    echo "Cannot find \"/root/provision/${shorthost}\".";
    exit;
fi;

if [[ -n ${2} ]]; then
    CPANEL_VERSION=${2};
fi;

if [[ -z ${IP} ]]; then
    echo "No IP specified. Please provide an IP in \"/root/provision/${shorthost}/provision-${shorthost}.conf\"";
    exit;
fi;

# Stop and remove the VM if its running
if virsh list --all | grep -q ${shorthost}; then 
    virsh destroy ${shorthost}.${DOMAIN};
    virsh undefine ${shorthost}.${DOMAIN};
fi;

# Run pre prov actions
if [[ -x /root/provision/${shorthost}/pre-provision-${shorthost}.sh ]]; then
    echo "Executing \"/root/provision/${shorthost}/pre-provision-${shorthost}.sh\"";
    /root/provision/${shorthost}/pre-provision-${shorthost}.sh;
fi;

# Kickstart the VM
virt-install -n ${shorthost}.${DOMAIN} -r ${RAM} --os-type=linux --os-variant=${OS} --vcpus=${CPUS} --cpu host --controller type=scsi,model=virtio-scsi --disk path=${DISK_PATH}/${shorthost}.${DOMAIN}.qcow2,format=qcow2,bus=scsi,size=${DISK_SIZE} --network bridge=${BRIDGE},model=virtio,mac=${MAC} --nographics -l ${LOCATION} -x "ks=http://kickstart.hostgator.com/ks_shared.php?partition=${PART_TYPE}&initial=${INITIAL_TYPE}&cp_ver=${CPANEL_VERSION}${EXTRA_KS_ARGS} ksdevice=${DEVICE} ip=${IP} netmask=${NETMASK} gateway=${GATEWAY} dns=${DNS} hostname=${shorthost}.${DOMAIN} console=ttyS0,115200" --noautoconsole || exit;


# Wait for kickstart to complete
echo -n "Kickstarting OS ...";
while virsh list | grep -qP "\b${shorthost}.${DOMAIN}\b"; do
    echo -n ".";
    sleep 10;
done;
echo "Done";


# Start the VM
echo -n "Starting VM ...";
sleep 5;
virsh start ${shorthost}.${DOMAIN};
# Copy in puppet keys, WHM accesshash, and install rsync
while ! ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no ${IP} "echo true" 2> /dev/null | grep -q true; do
    echo -n ".";
    sleep 10;
done;
echo "Done"
ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no ${IP} "mkdir -p /var/lib/puppet/ssl/public_keys; mkdir -p /var/lib/puppet/ssl/private_keys ; yum install rsync -y";
if [[ -e /root/provision/${shorthost}/${shorthost}.${DOMAIN}.pem-priv ]]; then
    rsync -avPe "ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no" /root/provision/${shorthost}/${shorthost}.${DOMAIN}.pem-priv ${IP}:/var/lib/puppet/ssl/private_keys/${shorthost}.${DOMAIN}.pem;
fi;
if [[ -e /root/provision/${shorthost}/${shorthost}.${DOMAIN}.pem-pub ]]; then
    rsync -avPe "ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no" /root/provision/${shorthost}/${shorthost}.${DOMAIN}.pem-pub ${IP}:/var/lib/puppet/ssl/public_keys/${shorthost}.${DOMAIN}.pem;
fi;
if [[ -e /root/provision/${shorthost}/.accesshash ]]; then
    rsync -avPe "ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no" /root/provision/${shorthost}/.accesshash ${IP}:/root/
fi;
if [[ -e /root/provision/${shorthost}/motd ]]; then
    rsync -avPe "ssh -o UserKnownHostsFile=/dev/null -o StrictHostKeyChecking=no" /root/provision/${shorthost}/motd ${IP}:/root/.motd
fi;

echo "Setting root password"
/root/bin/setpass.pl ${IP}

# Run post prov actions
if [[ -x /root/provision/${shorthost}/post-provision.${shorthost}.sh ]]; then
    echo "Executing \"/root/provision/${shorthost}/post-provision.${shorthost}.sh\"";
    /root/provision/${shorthost}/post-provision.${shorthost}.sh;
fi;
